;;
;; readline module
;;
;; by: YOKOTA Hiroshi

(define-module readline
  (use gauche.uvector)
  (use gauche.vport)
  (export make-readline-input-port
	  rl-readline
	  rl-set-application-name!
	  rl-add-history! rl-read-history! rl-write-history rl-stifle-history!
	  rl-set-prompt!
	  rl-set-prompt-state!
	  rl-set-completion-list-function!
	  rl-set-basic-word-break-characters!
	  rl-set-basic-quote-characters!
	  ))
(select-module readline)

(dynamic-load "readline")

(define (make-readline-input-port)
  (make <buffered-input-port>
    :fill (lambda (vec)
	    (let ((line #f))

	      (set! line (rl-readline rl-prompt))
	      (rl-set-prompt-state! 'ps2)

	      (if (string? line)
		  (let* ((rlvec (string->u8vector line))
			 (len   (u8vector-length  rlvec)))

		    (if (not (= (string-length line) 0))
			(rl-add-history! line))

		    (u8vector-copy! vec 0 rlvec)

		    ;; Add #\newline for line separator. Then add
		    ;; 1 to length and exits this function with
		    ;; length value.
		    (u8vector-set! vec len (char->integer #\newline))
		    (+ len 1))

		  0)))))

;; prompt strings
(define rl-prompt  "")
(define rl-prompt1 "readline> ")
(define rl-prompt2 "")

(define (rl-set-prompt! . args)
  (set! rl-prompt1 (get-keyword :ps1 args "readline> "))
  (set! rl-prompt2 (get-keyword :ps2 args "")))
(define (rl-set-prompt-state! state)
  (set! rl-prompt
	(cond ((eq? state 'ps1) rl-prompt1)
	      ((eq? state 'ps2) rl-prompt2)
	      (else             "???"))))



(provide "readline")

;; end
